import http from 'k6/http'
import { check } from 'k6'

import { graphQLHeaders, graphQLApiUrl, oryLoginUrl, oryUserIdentifier, oryUserPassword } from './const.js'
import { checkFailureRate, timeToFirstByte } from './main.js'

export function loginOryGetAuthHeaders() {
  const oryLoginFlowIdUrl = `${oryLoginUrl}/api`
  const resOryLoginFlowIdUrl = http.get(oryLoginFlowIdUrl)
  const oryLoginFlowId = JSON.parse(resOryLoginFlowIdUrl.body).id

  const loginUrl = `${oryLoginUrl}?flow=${oryLoginFlowId}`
  const resLogin = http.post(loginUrl, {
    identifier: oryUserIdentifier,
    password: oryUserPassword,
    method: 'password',
  }).body
  const resLoginBody = JSON.parse(resLogin)
  return {
    graphQLAuthenticatedUserHeaders: getGraphQLAuthenticatedUserHeaders(resLoginBody.session_token),
    userId: resLoginBody.session.identity.id,
  }
}

export function addBatchRequest(query, variables, tags, headers) {
  return [
    'POST',
    graphQLApiUrl,
    JSON.stringify({ query, variables: variables || {} }),
    {
      headers: headers !== undefined ? headers : graphQLHeaders,
      tags,
    },
  ]
}

export function createPostRequest(query, variables, tags, headers) {
  return http.post(
    graphQLApiUrl,
    JSON.stringify({
      query,
      variables: variables || {},
    }),
    {
      headers: headers !== undefined ? headers : graphQLHeaders,
      tags,
    },
  )
}

export function checkResponse(response) {
  const checkResponse = check(response, {
    'is status 200': (r) => r.status === 200,
    'has no errors': (r) => JSON.parse(r.body) && JSON.parse(r.body).errors === undefined,
  })

  if (JSON.parse(response.body) && JSON.parse(response.body).errors !== undefined) {
    console.warn('errors in response:', JSON.parse(response.body).errors)
  }

  checkFailureRate.add(!checkResponse)
  timeToFirstByte.add(response.timings.waiting, {
    ttfbURL: response.url,
  })
}

export function getGraphQLAuthenticatedUserHeaders(sessionToken) {
  return {
    'X-Disable-Sentry': 'true',
    'Content-Type': 'application/json',
    Authorization: `Bearer ${sessionToken}`,
  }
}

export function isResponseValid(response) {
  return response.status === 200 && JSON.parse(response.body) && JSON.parse(response.body).data
}

export function generateRandomString(prefix = '') {
  return `${prefix} ${(Math.random() + 1).toString(36).substring(2)}`
}
